@@ -3,24 +3,26 @@ import json
 import os
 import shutil
 import sys
-from pathlib import Path
-from typing import List
+from pathlib import Path, PosixPath
+from typing import List, Union
 
 import pkg_resources
 import requests  # type: ignore
 import tomli
 import uvicorn  # type: ignore
 from fastapi import FastAPI, WebSocket
-from fastapi.responses import Response
 from fastapi.staticfiles import StaticFiles
 
 from zeno.classes import (
+    FilterPredicate,
+    FilterPredicateGroup,
+    HistogramRequest,
     MetricKey,
     MirrorProject,
     Report,
     Slice,
     StatusResponse,
-    ZenoColumn,
+    TableRequest,
     ZenoSettings,
     ZenoVariables,
 )
@@ -136,7 +138,7 @@ def main():
 
     if "data_path" not in args:
         args["data_path"] = ""
-    else:
+    elif not args["data_path"].startswith("http"):
         args["data_path"] = Path(
             os.path.realpath(os.path.join(toml_path, args["data_path"]))
         )
@@ -166,6 +168,11 @@ def main():
     if "samples" not in args:
         args["samples"] = 30
 
+    if "editable" not in args:
+        args["editable"] = True
+    else:
+        args["editable"] = args["editable"]
+
     if "port" not in args:
         args["port"] = 8000
 
@@ -190,13 +197,14 @@ def run_zeno(args):
         data_path=args["data_path"],
         label_path=args["label_path"],
         cache_path=args["cache_path"],
+        editable=args["editable"],
     )
     zeno.start_processing()
 
     app = FastAPI(title="Frontend API")
     api_app = FastAPI(title="Backend API")
 
-    if args["data_path"] != "":
+    if args["data_path"] != "" and isinstance(args["data_path"], PosixPath):
         app.mount("/data", StaticFiles(directory=args["data_path"]), name="static")
     if args["label_path"] != "":
         app.mount("/labels", StaticFiles(directory=args["label_path"]), name="static")
@@ -223,8 +231,12 @@ def run_zeno(args):
             id_column=zeno.id_column,
             label_column=zeno.label_column,
             data_column=zeno.data_column,
+            data_origin="/data/"
+            if isinstance(args["data_path"], PosixPath)
+            else args["data_path"],
             metadata_columns=zeno.columns,
             samples=args["samples"],
+            totalSize=zeno.df.shape[0],
         )
 
     @api_app.get("/initialize", response_model=ZenoVariables)
@@ -244,26 +256,38 @@ def run_zeno(args):
     def get_reports():
         return json.dumps(zeno.get_reports())
 
-    @api_app.post("/results")
-    def get_results(reqs: List[MetricKey]):
-        return json.dumps(zeno.get_results(reqs))
-
-    @api_app.post("/table")
-    def get_table(columns: List[ZenoColumn]):
-        return Response(zeno.get_table(columns))
-
     @api_app.post("/set-folders")
     def set_folders(folders: List[str]):
         zeno.set_folders(folders)
 
-    @api_app.post("/set-slices")
-    def set_slices(slices: List[Slice]):
-        zeno.set_slices(slices)
-
     @api_app.post("/set-reports")
     def update_reports(reqs: List[Report]):
         zeno.set_reports(reqs)
 
+    @api_app.post("/get-filtered-ids")
+    def get_filtered_ids(req: List[Union[FilterPredicateGroup, FilterPredicate]]):
+        return json.dumps(zeno.get_filtered_ids(req))
+
+    @api_app.post("/get-filtered-table")
+    def get_filtered_table(req: TableRequest):
+        return zeno.get_filtered_table(req)
+
+    @api_app.post("/calculate-histograms")
+    def calculate_histograms(req: HistogramRequest):
+        return json.dumps(zeno.calculate_histograms(req))
+
+    @api_app.post("/create-new-slice")
+    def create_new_slice(req: Slice):
+        zeno.create_new_slice(req)
+
+    @api_app.post("/delete-slice")
+    def delete_slice(slice_name: List[str]):
+        zeno.delete_slice(slice_name[0])
+
+    @api_app.post("/get-metrics-for-slices")
+    def get_metrics_for_slices(reqs: List[MetricKey]):
+        return json.dumps(zeno.get_metrics_for_slices(reqs))
+
     @api_app.post("/mirror/project")
     def mirror_project(req: MirrorProject):
         # not specified or entire data frame just use the cache
